#!/usr/bin/perl
use strict;
use Bio::SeqIO;                		#Index Fasta files

##########################################################
##         Created by Pablo MIER on June 2019.				    ##
##   Copyright © 2019 Pablo MIER. All rights reserved.  ##
##########################################################

#Generate results

my $proteome = $ARGV[0]; chomp $proteome;
my $name = $ARGV[1]; chomp $name; $name =~ s/\.//gi; $name =~ s/ //gi;

if (! -e "$proteome") { print "A valid file must be provided\n\nUSAGE:\nperl PolyQ_Scanner.pl FILE NAME\n\n"; exit;}
if (length($name)< 1) { $name = "NAME"; }

my $aa = "Q";
my $polyxregion = "lengths.txt";						open (OUT1, ">>$polyxregion");			print OUT1 "threshold\tlength\timpurities\tglutamines\tproteome\n";
my $polyximpurities = "impurities.txt";		open (OUT2, ">>$polyximpurities");	print OUT2 "threshold\tpurity\tvalue\tproteome\n";
my @min = (4,4,6,6,8);	my @num = (4,6,6,8,10);			
my @ppp = (0,0,0,0,0);	my @iii = (0,0,0,0,0);			

my $multifasta = Bio::SeqIO->new( -file  => "<$proteome" , '-format' => 'fasta');
while (my $seq = $multifasta->next_seq()) { 
	my $na = $seq->display_id;		my $ss = $seq->seq;		my $length = length($ss);	my $ya = 0;		my $initial = 0; my $final;
	for (my $iter = 0; $iter < 5; $iter++) {
		for (my $sl = 0; $sl <= $length; $sl++) {
			my $win6 = substr($ss,$sl,$num[$iter]);	
			my $c6 = ($win6 =~ s/$aa//gi); 				####		POLYQ, LENGTH = $num[$iter], MINIMUM NUMBER OF Q = $min[$iter]
			if ($c6 >= $min[$iter]) { 	if ($ya == 0) {		$initial = $sl;		}	$ya = 1;	}	
		else {		
				if ($ya == 1) {	#End of polyQ, if it was open; in that case, print
					my $ant = $initial-10; my $lenant = 10;	if ($ant < 1) { $ant = 0; $lenant = $initial; }	
					my $laew = $sl-$initial-1+$num[$iter];	my $polyx = substr($ss, $initial, $laew);
					my $wq = "-"; my $rest = 0;
					while ($wq ne $aa) {		
						my $bf = substr($polyx, 0, 1);	if ($bf eq $aa) { $wq = $aa; }	else {	$polyx = substr($polyx, 1); 	$rest++	}
					}
					my $wq2 = "-"; my $rest2 = 0;
					while ($wq2 ne $aa) {	
						my $leng = length($polyx);				my $oie = $leng - 1;	my $bf2 = substr($polyx, $oie, 1);
						if ($bf2 eq $aa) { $wq2 = $aa; }	else {	$polyx = substr($polyx,0,$oie);	$rest2++; }
					}
					my $lenpoly = length($polyx);		$initial = 0;	$ya = 0;	my $xdr = $polyx;		my $mgf = ($xdr =~ s/$aa//gi);
					my $impur = $lenpoly-$mgf;			print OUT1 "$min[$iter]/$num[$iter]\t$lenpoly\t$impur\t$mgf\t$name\n";
					if ($impur > 0) { $iii[$iter]++; } else { $ppp[$iter]++}
}	}	}	}
}

for (my $iter = 0; $iter < 5; $iter++) {
	print OUT2 "$min[$iter]/$num[$iter]\tpure\t$ppp[$iter]\t$name\n";
	print OUT2 "$min[$iter]/$num[$iter]\timpure\t$iii[$iter]\t$name\n";
}
close OUT1; close OUT2;

#Generate figure

open (OUT, ">commands_file.r");		
print OUT "
library(reshape2); library(ggplot2); require(gridExtra); source(\"http://peterhaschke.com/Code/multiplot.R\")\n
data1 <- read.table(\"lengths.txt\",header=TRUE)\n
p1 <- ggplot(data1) + 
	geom_boxplot(aes_string(x=\"as.factor(threshold)\", y=\"length\"), na.rm=TRUE,outlier.size = 0.5) + 
	labs(fill = \"\", x = \"Threshold\",y = \"poly$aa length\",title=\"b)\") +
	theme_classic() + scale_y_continuous(breaks=c(5, 10, 20,40,60,100,200), trans = 'log') +
	theme(axis.text=element_text(size=8), axis.text.x = element_text(angle = 90)) +
	theme(strip.text = element_text(size=7,lineheight=5.0))\n
dat2 = read.table('impurities.txt',sep='\t',header=TRUE,dec=',')\n
dat2\$purity <- factor(dat2\$purity, levels=c(\"pure\", \"impure\"))\n
p2 <- ggplot(data=dat2, aes(x=threshold, y=value, fill=purity)) + geom_bar(stat=\"identity\", position=position_dodge()) +
	labs(x = \"Threshold\",y = \"Number of poly$aa\",title=\"a)\",fill=\"\") +
	theme_classic() + theme(axis.text=element_text(size=8), axis.text.x = element_text(angle = 90)) +
	theme(legend.position = c(.99, 0.95),legend.justification = c(\"right\", \"top\"),
	legend.box.just = \"right\", legend.margin = margin(2, 2, 2, 2),legend.text = element_text(size = 8),
	legend.title = element_text(size = 8),legend.key.size = unit(0.5,\"line\")) 	+
	geom_text(aes(label = value, y = value), size = 2.5, position = position_dodge(width=1), angle = 90) +
	theme(strip.text = element_text(size=7,lineheight=5.0))\n
pdf('plotPoly$aa-$name.pdf');grid.arrange(p2,p1, ncol=2, top = \"$name\");dev.off()\n
\n";
close OUT;
`R --save < commands_file.r`;	
unlink "commands_file.r";		unlink $polyxregion; unlink $polyximpurities;

exit;
